import React, { useState, useCallback, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { useUsMap } from './useUsMap';
import { useCities } from './useCities';
import { Marks } from './Marks';
import { Pies } from './Pies';
import { ColorLegend } from './ColorLegend';
import { csv, scaleSqrt, scaleOrdinal, max, format, extent, scaleLinear
        ,select, pie, arc, nest, geoAlbers} from 'd3';

const width = 960;
const height = 600;
const margin = { top: 20, right: 220, bottom: 20, left: 90 };
const xAxisLabelOffset = 45;
const yAxisLabelOffset = 20;
const fadeOpacity = 0.1;
const circleRadius = 6;
const legendOffset = 185;
const pieStrokeWidth = 1;
const innerRadius = 25;
const outerRadius = 50;
const innerHeight = height - margin.top - margin.bottom;
const innerWidth = width - margin.left - margin.right;
  
const colorValue = d => d.MainAlcohol;
const colorLegendLabel = 'Alcohol';  
const tooltipIngredient = d => d['data']['key'];
const tooltipCount = d => d['data']['value'];
const pieValue = d => d.value;

const xValue = d => d.Longitude;
const yValue = d => d.Latitude;

const projection = geoAlbers();

const [sfX, sfY] = projection([-122.4194, 37.7749]);  
const [nyX, nyY] = projection([-73.9249, 40.6943]);  
const [houstonX, houstonY] = projection([-95.3905, 29.7868]); 
const [laX, laY] = projection([-118.2437, 34.0522]);  
const [chiX, chiY] = projection([-87.6861, 41.8373]);   
const [miaX, miaY] = projection([-80.2102, 25.784+2]);  
const [seaX, seaY] = projection([-122.3244, 47.6211-1]) 
const [denX, denY] = projection([-104.8759, 39.7621]);  

const App = () => {
  const usMap = useUsMap();
  let cities = useCities();
  const [hoveredValue, setHoveredValue] = useState(null);
  
  if (!usMap || !cities) {
    return <pre>Loading...</pre>;
  }


cities = cities
  .sort(function(x, y){return d3.ascending(x.MainAlcohol, y.MainAlcohol)})
  
const aggData =  nest()
    .key(colorValue)
    .rollup(ingredients => ingredients.length)
    .entries(cities)
		.sort(function(x, y){return d3.descending(x.value, y.value)});
  

const filteredData = cities.filter(d => hoveredValue === colorValue(d)); 
  
const sfData =  nest()
    .key(colorValue)
    .rollup(ingredients => ingredients.length)
    .entries(cities.filter(d => d.Location == "San Francisco")
    );
  
const sfPie = pie().value(pieValue)(sfData);
  

const nYData =  nest()
    .key(colorValue)
    .rollup(ingredients => ingredients.length)
    .entries(cities.filter(d => d.Location == "New York")
    );
     
const nyPie = pie()
			.value(pieValue)
  		(nYData);
  
const houstonData =  nest()
    .key(colorValue)
    .rollup(ingredients => ingredients.length)
    .entries(cities.filter(d => d.Location == "Houston")
    );

const houstonPie = pie()
			.value(pieValue)
  		(houstonData);
  
const laData =  nest()
    .key(colorValue)
    .rollup(ingredients => ingredients.length)
    .entries(cities.filter(d => d.Location == "Los Angeles")
    );

const laPie = pie()
			.value(pieValue)
  		(laData);
  
  let chiData =  nest()
    .key(colorValue)
    .rollup(ingredients => ingredients.length)
    .entries(cities.filter(d => d.Location == "Chicago")
    );

    const chiPie = pie()
			.value(pieValue)
  		(chiData);
  
    let denData =  nest()
    .key(colorValue)
    .rollup(ingredients => ingredients.length)
    .entries(cities.filter(d => d.Location == "Denver")
    );

    const denPie = pie()
			.value(pieValue)
  		(denData);
  
      let seaData =  nest()
    .key(colorValue)
    .rollup(ingredients => ingredients.length)
    .entries(cities.filter(d => d.Location == "Seattle")
    );

    const seaPie = pie()
			.value(pieValue)
  		(seaData);
  
      let miaData =  nest()
    .key(colorValue)
    .rollup(ingredients => ingredients.length)
    .entries(cities.filter(d => d.Location == "Miami")
    );

    const miaPie = pie()
			.value(pieValue)
  		(miaData);
  
  
  const xScale = scaleLinear()
    .domain(extent(cities, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(cities, yValue))
    .range([0, innerHeight]);

  const color = ['#a6cee3','#1f78b4','#b2df8a','#33a02c', '#fb9a99','#e31a1c','#fdbf6f','#ff7f00','#cab2d6','#6a3d9a'];
  const colorScale = scaleOrdinal()
    .domain(cities.map(colorValue))
    .range(color);

  
  return (
    <svg width={width} height={height}>
      <g transform={`translate(${innerWidth + legendOffset}, 60)`}>
          <text x={xAxisLabelOffset} y={-yAxisLabelOffset} className="axis-label" textAnchor="middle">
          {colorLegendLabel}
          </text>
          <ColorLegend
            tickSpacing={22}
            tickSize={10}
            tickTextOffset={12}
            tickSize={circleRadius}
            colorScale={colorScale}
            onHover={setHoveredValue}
            hoveredValue={hoveredValue}
            fadeOpacity={fadeOpacity}
            aggData={aggData}
          />
        </g>
      
      <g transform={`translate(-50, 10)`}>

      	<Marks 
        usMap={usMap} cities = {cities}
  			colorScale={colorScale}
  			colorValue={colorValue}
          />
      	<Marks 
        usMap={usMap} cities = {cities}
  			colorScale={colorScale}
  			colorValue={colorValue}
          />
      <g opacity={hoveredValue ? fadeOpacity : 1}>
       <Pies
         statePie={sfPie}
         xValue = {sfX}
         yValue = {sfY}
    		 tooltipIngredient={tooltipIngredient}
         tooltipCount={tooltipCount}
          innerRadius = {innerRadius}
          outerRadius ={outerRadius}
          pieStrokeWidth={pieStrokeWidth}
          colorScale = {colorScale}
					hoveredValue ={hoveredValue}
          isFilteredPie = {false}        
         />
        <Pies
         statePie={nyPie}
         xValue = {nyX}
         yValue = {nyY}
    		 tooltipIngredient={tooltipIngredient}
         tooltipCount={tooltipCount}
          innerRadius = {innerRadius}
          outerRadius ={outerRadius}
          pieStrokeWidth={pieStrokeWidth}
          colorScale = {colorScale}
					hoveredValue ={hoveredValue}
          isFilteredPie = {false}          
         />
        <Pies
         statePie={chiPie}
         xValue = {chiX}
         yValue = {chiY}
    		 tooltipIngredient={tooltipIngredient}
         tooltipCount={tooltipCount}
          innerRadius = {innerRadius}
          outerRadius ={outerRadius}
          pieStrokeWidth={pieStrokeWidth}
          colorScale = {colorScale}
					hoveredValue ={hoveredValue}
          isFilteredPie = {false}          
         />
        <Pies
         statePie={houstonPie}
         xValue = {houstonX}
         yValue = {houstonY}
    		 tooltipIngredient={tooltipIngredient}
         tooltipCount={tooltipCount}
          innerRadius = {innerRadius}
          outerRadius ={outerRadius}
          pieStrokeWidth={pieStrokeWidth}
          colorScale = {colorScale}
					hoveredValue ={hoveredValue}
          isFilteredPie = {false}          
         />
        <Pies
         statePie={laPie}
         xValue = {laX}
         yValue = {laY}
    		 tooltipIngredient={tooltipIngredient}
         tooltipCount={tooltipCount}
          innerRadius = {innerRadius}
          outerRadius ={outerRadius}
          pieStrokeWidth={pieStrokeWidth}
          colorScale = {colorScale}
          hoveredValue ={hoveredValue}
          isFilteredPie = {false}
         />
                    <Pies
         statePie={seaPie}
         xValue = {seaX}
         yValue = {seaY}
    		 tooltipIngredient={tooltipIngredient}
         tooltipCount={tooltipCount}
          innerRadius = {innerRadius}
          outerRadius ={outerRadius}
          pieStrokeWidth={pieStrokeWidth}
          colorScale = {colorScale}
					hoveredValue ={hoveredValue}
          isFilteredPie = {false}
         />
                  <Pies
         statePie={miaPie}
         xValue = {miaX}
         yValue = {miaY}
    		 tooltipIngredient={tooltipIngredient}
         tooltipCount={tooltipCount}
          innerRadius = {innerRadius}
          outerRadius ={outerRadius}
          pieStrokeWidth={pieStrokeWidth}
          colorScale = {colorScale}
					hoveredValue ={hoveredValue}
          isFilteredPie = {false}
         />
                  <Pies
         statePie={denPie}
         xValue = {denX}
         yValue = {denY}
    		 tooltipIngredient={tooltipIngredient}
         tooltipCount={tooltipCount}
          innerRadius = {innerRadius}
          outerRadius ={outerRadius}
          pieStrokeWidth={pieStrokeWidth}
          colorScale = {colorScale}
					hoveredValue ={hoveredValue}
          isFilteredPie = {false}
         />  
      </g>
        <g>
        <Pies
         statePie={sfPie}
         xValue = {sfX}
         yValue = {sfY}
    		 tooltipIngredient={tooltipIngredient}
         tooltipCount={tooltipCount}
          innerRadius = {innerRadius}
          outerRadius ={outerRadius}
          pieStrokeWidth={pieStrokeWidth}
          colorScale = {colorScale}
					hoveredValue ={hoveredValue}
          isFilteredPie = {true}
         />
        <Pies
         statePie={nyPie}
         xValue = {nyX}
         yValue = {nyY}
    		 tooltipIngredient={tooltipIngredient}
         tooltipCount={tooltipCount}
          innerRadius = {innerRadius}
          outerRadius ={outerRadius}
          pieStrokeWidth={pieStrokeWidth}
          colorScale = {colorScale}
					hoveredValue ={hoveredValue}
          isFilteredPie = {true}
         />
        <Pies
         statePie={chiPie}
         xValue = {chiX}
         yValue = {chiY}
    		 tooltipIngredient={tooltipIngredient}
         tooltipCount={tooltipCount}
          innerRadius = {innerRadius}
          outerRadius ={outerRadius}
          pieStrokeWidth={pieStrokeWidth}
          colorScale = {colorScale}
					hoveredValue ={hoveredValue}
          isFilteredPie = {true}
         />
        <Pies
         statePie={houstonPie}
         xValue = {houstonX}
         yValue = {houstonY}
    		 tooltipIngredient={tooltipIngredient}
         tooltipCount={tooltipCount}
          innerRadius = {innerRadius}
          outerRadius ={outerRadius}
          pieStrokeWidth={pieStrokeWidth}
          colorScale = {colorScale}
					hoveredValue ={hoveredValue}
          isFilteredPie = {true}
         />
        <Pies
         statePie={laPie}
         xValue = {laX}
         yValue = {laY}
    		 tooltipIngredient={tooltipIngredient}
         tooltipCount={tooltipCount}
          innerRadius = {innerRadius}
          outerRadius ={outerRadius}
          pieStrokeWidth={pieStrokeWidth}
          colorScale = {colorScale}
					hoveredValue ={hoveredValue}
          isFilteredPie = {true}
         />
            <Pies
         statePie={seaPie}
         xValue = {seaX}
         yValue = {seaY}
    		 tooltipIngredient={tooltipIngredient}
         tooltipCount={tooltipCount}
          innerRadius = {innerRadius}
          outerRadius ={outerRadius}
          pieStrokeWidth={pieStrokeWidth}
          colorScale = {colorScale}
					hoveredValue ={hoveredValue}
          isFilteredPie = {true}
         />
                  <Pies
         statePie={miaPie}
         xValue = {miaX}
         yValue = {miaY}
    		 tooltipIngredient={tooltipIngredient}
         tooltipCount={tooltipCount}
          innerRadius = {innerRadius}
          outerRadius ={outerRadius}
          pieStrokeWidth={pieStrokeWidth}
          colorScale = {colorScale}
					hoveredValue ={hoveredValue}
          isFilteredPie = {true}
         />
                  <Pies
         statePie={denPie}
         xValue = {denX}
         yValue = {denY}
    		 tooltipIngredient={tooltipIngredient}
         tooltipCount={tooltipCount}
          innerRadius = {innerRadius}
          outerRadius ={outerRadius}
          pieStrokeWidth={pieStrokeWidth}
          colorScale = {colorScale}
					hoveredValue ={hoveredValue}
          isFilteredPie = {true}
         />      
      </g>
     </g>
    </svg>
  );
};
const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
